@@ -9,6 +9,7 @@ from talos.thrift.topic.ttypes import TopicTalosResourceName
 from talos.admin.TalosAdmin import TalosAdmin
 from talos.client.TalosClientConfig import TalosClientConfig
 from talos.client.TalosClientFactory import TalosClientFactory
+from talos.client.ScheduleInfoCache import ScheduleInfoCache
 from talos.utils import Utils
 from talos.utils.Utils import ReadWriteLock
 from talos.utils.FalconWriter import FalconWriter
@@ -120,6 +121,12 @@ class TalosConsumer(object):
             self.partitionCheckPoint = partitionCheckPoint
         else:
             self.partitionCheckPoint = dict()
+        # get scheduleInfo
+        self.scheduleInfoCache = ScheduleInfoCache().get_schedule_info_cache(
+            topicTalosResourceName=self.topicTalosResourceName.topicTalosResourceName,
+            talosClientConfig=consumerConfig,
+            messageClient=self.talosClientFactory.new_message_client(),
+            talosClientFactory=self.talosClientFactory)
         self.falconWriter = FalconWriter(falconUrl=consumerConfig.get_falcon_url())
         self.isActive = True
 
@@ -616,5 +623,6 @@ class TalosConsumer(object):
         self.shut_down_fetcher()
         self.renewScheduleExecutor.cancel()
         self.consumerMonitorThread.cancel()
+        self.scheduleInfoCache.shut_down(self.topicTalosResourceName)
         self.isActive = False
         logger.info("Worker: " + self.workerId + " shutdown.")
