@@ -8,6 +8,8 @@
 from talos.client.TalosClientConfig import TalosClientConfig
 from talos.client.compression.Compression import Compression
 from talos.client.TalosClientFactory import TalosClientFactory
+from talos.client.ScheduleInfoCache import ScheduleInfoCache
+from talos.thrift.transport.TSocket import TTransportException
 from talos.thrift.topic.ttypes import TopicAndPartition
 from talos.thrift.topic.TopicService import GetDescribeInfoRequest
 from talos.thrift.message import MessageService
@@ -16,6 +18,7 @@ from talos.thrift.message.ttypes import PutMessageRequest
 from atomic import AtomicLong
 from talos.utils import Utils
 import logging
+import traceback
 
 
 class SimpleProducer(object):
@@ -24,6 +27,7 @@ class SimpleProducer(object):
     producerConfig = TalosClientConfig
     topicAndPartition = TopicAndPartition
     messageClient = MessageService.Iface
+    talosClientFactory = TalosClientFactory
     requestId = AtomicLong
     clientId = str
     isActive = bool
@@ -54,6 +58,10 @@ class SimpleProducer(object):
             self.requestId = requestId
         else:
             self.requestId = AtomicLong(1)
+        self.scheduleInfoCache = ScheduleInfoCache().get_schedule_info_cache(
+            topicTalosResourceName=self.topicAndPartition.topicTalosResourceName,
+            talosClientConfig=self.producerConfig, messageClient=self.messageClient,
+            talosClientFactory=self.talosClientFactory)
         self.isActive = True
 
     def get_topic_info(self, topicClient=None, topicName=None, partitionId=None):
@@ -97,14 +105,38 @@ class SimpleProducer(object):
         putMessageRequest = PutMessageRequest(self.topicAndPartition, messageBlockList,
                                               len(msgList), requestSequenceId)
         try:
-            putMessageResponse = self.messageClient.put_message(putMessageRequest)
-        except Exception as e:
-            self.logger.error("put message request failed." + e.message)
-            raise e
+            putMessageResponse = self.scheduleInfoCache.get_or_create_message_client(
+                topicAndPartition=self.topicAndPartition).put_message(putMessageRequest)
+        except TTransportException as tTransportException:
+            if not self.scheduleInfoCache and self.scheduleInfoCache.get_is_auto_location():
+                self.logger.warn(
+                    "can't connect to the host directly, refresh scheduleInfo and retry using url. "
+                    + "The exception message is :" + tTransportException.message +
+                    ". Ignore this if not frequently.")
+                self.scheduleInfoCache.update_schedule_info_cache()
+                putMessageResponse = self.messageClient.put_message(putMessageRequest)
+            else:
+                self.logger.error(
+                    "put message request failed." + str(traceback.format_exc()))
+                raise tTransportException
+
+        # update scheduleInfocache when request have been transfered and talos auto location was set up
+        if putMessageResponse.isTransfer is not None and putMessageResponse.isTransfer and self.scheduleInfoCache is not None \
+                and self.scheduleInfoCache.get_is_auto_location():
+            self.logger.info(
+                "request has been transfered when talos auto location set up, refresh scheduleInfo")
+            self.scheduleInfoCache.update_schedule_info_cache()
 
     def _compress_message_list(self, msgList=None):
         return Compression().compress(msgList, self.producerConfig.get_compression_type())
 
+    def shut_down(self):
+        if not self.isActive:
+            self.logger.info("SimpleProducer which clientId is: " + str(self.clientId)
+                             + " is already shutdown, don't do it again.")
+            return
 
+        self.scheduleInfoCache.shut_down(self.topicAndPartition.topicTalosResourceName)
+        self.isActive = False
 
 
